﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Random;

namespace HeuristicLab.Encodings.ParameterConfigurationEncoding {
  /// <summary>
  /// An operator which crosses two integer values. TODO
  /// </summary>
  [Item("NormalIntValueCrossover", "An operator which crosses two integer values. TODO")]
  [StorableClass]
  public class NormalIntValueCrossover : Item, IIntValueCrossover {
    #region Constructors and Cloning
    public NormalIntValueCrossover() : base() { }

    [StorableConstructor]
    protected NormalIntValueCrossover(bool deserializing) : base(deserializing) { }
    protected NormalIntValueCrossover(NormalIntValueCrossover original, Cloner cloner) : base(original, cloner) { }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new NormalIntValueCrossover(this, cloner);
    }
    #endregion

    public void Apply(IRandom random, IntValue value, IntValue other, IntValueRange range) {
      ApplyStatic(random, value, other, range);
    }

    public static void ApplyStatic(IRandom random, IntValue value, IntValue other, IntValueRange range) {
      var N = new NormalDistributedRandom(random, value.Value, Math.Abs(value.Value - other.Value) / 3);

      int offspring;
      do {
        offspring = (int)N.NextDouble();
        offspring = range.ApplyStepSize(offspring);
      } while (!range.IsInRange(offspring));

      value.Value = offspring;
    }
  }
}
